#ifndef MYX_REDIS_BASE_INL_HPP_
#define MYX_REDIS_BASE_INL_HPP_

#pragma once

#ifndef MYXLIB_HEADER_ONLY
#include <myx/redis/base.hpp>
#endif

#include <myx/base/config.hpp>

#include <QTimer>

namespace myx {

namespace redis {

/*!
 * \brief Создает объект класса
 * \param host Имя хоста сервера
 * \param port Номер порта сервера
 * \param parent Родительский объект
 */
inline Base::Base( QObject* parent ) :
	QObject     { parent },
	m_readTimer ( new QTimer( this ) ),
	m_writeTimer( new QTimer( this ) )
{
	m_readTimer->setSingleShot( true );
	m_writeTimer->setSingleShot( true );
	m_readTimer->setInterval( rwTimeout() );
	m_writeTimer->setInterval( rwTimeout() );
	connect( m_readTimer, &QTimer::timeout, this, [this]() {
		if ( ( ( m_connectionFlags & kConnected ) != 0 ) && ( ( m_connectionFlags & kRead ) != 0 ) )
		{
			m_connectionFlags -= kRead;
			Q_EMIT connectionStateChanged( m_connectionFlags );
		}
	} );
	connect( m_writeTimer, &QTimer::timeout, this, [this]() {
		if ( ( ( m_connectionFlags & kConnected ) != 0 ) && ( ( m_connectionFlags & kWrite ) != 0 ) )
		{
			m_connectionFlags -= kWrite;
			Q_EMIT connectionStateChanged( m_connectionFlags );
		}
	} );
}


inline int Base::rwTimeout() const
{
	return( kRWTimeout );
}


inline QString Base::host() const
{
	return( m_host );
}


/*!
 * \brief Задает адрес хоста
 * \param host Адрес
 */
inline void Base::setHost( const QString& host )
{
	m_host = host;
}


inline quint16 Base::port() const
{
	return( m_port );
}


inline void Base::setPort( quint16 port )
{
	m_port = port;
}


inline int Base::connectionFlags() const
{
	return( m_connectionFlags );
}


inline void Base::setConnectionFlags( int connectionFlags )
{
	m_connectionFlags = connectionFlags;
}


/*!
 * \brief Возвращает признак установленного соединения
 * \return Признак установленного соединения
 */
inline bool Base::isConnected() const
{
	return( ( m_connectionFlags & kConnected ) != 0 );
}


/*!
 * \brief Добавляет к состоянию флаг Read
 */
inline void Base::onReadyRead()
{
	if ( ( ( m_connectionFlags & kConnected ) != 0 ) && ( ( m_connectionFlags & kRead ) == 0 ) )
	{
		m_connectionFlags += kRead;
		Q_EMIT connectionStateChanged( m_connectionFlags );
	}
	m_readTimer->start();
}


/*!
 * \brief Добавляет к состоянию флаг Read
 */
inline void Base::onBytesWritten()
{
	if ( ( ( m_connectionFlags & kConnected ) != 0 ) && ( ( m_connectionFlags & kWrite ) == 0 ) )
	{
		m_connectionFlags += kWrite;
		Q_EMIT connectionStateChanged( m_connectionFlags );
	}
	m_writeTimer->start();
}


/*!
 * \brief Выставляет флаг ошибки соединения
 */
inline void Base::onSocketError()
{
	Q_EMIT connectionStateChanged( m_connectionFlags = kError );
}

} // namespace redis

} // namespace myx

#endif // ifndef MYX_REDIS_BASE_INL_HPP_
